# Generated by Django 2.0.6 on 2018-08-24 14:21

from django.db import migrations


def create_userplans(apps, schema_editor):
    from plans.models import UserPlan
    UserPlan.create_for_users_without_plan()


class Migration(migrations.Migration):

    dependencies = [
        ('plans', '0003_make_plans_unique'),
    ]

    operations = [
        migrations.RunPython(create_userplans, reverse_code=migrations.RunPython.noop),
    ]
